==> gnustep-base_1.30.0.orig.tar.gz: gnustep-base-1.30.0/Tests/base/NSOperation/concurrent.m <==
#import <Foundation/NSArray.h>
#import <Foundation/NSKeyValueObserving.h>
#import <Foundation/NSLock.h>
#import <Foundation/NSOperation.h>
#import <Foundation/NSThread.h>
#import <Foundation/NSNotification.h>
#import <Foundation/NSAutoreleasePool.h>
#import "ObjectTesting.h"

// concurrent operation
@interface MyOperation : NSOperation
{
  int calculation;

  BOOL executing;
  BOOL finished;
}
- (void) completeOperation;
@end

@implementation MyOperation

- (id) initWithValue: (int)aVal
{
  self = [super init];
  if (self) {
    calculation = aVal;
    executing = NO;
    finished = NO;
  }
  return self;
}

- (BOOL) isConcurrent { return YES; }
- (BOOL) isExecuting { return executing; }
- (BOOL) isFinished { return finished; }
- (int) getCalculation { return calculation; }

- (void) main
{
  NSAutoreleasePool* pool = [[NSAutoreleasePool alloc] init];

  [self beginOperation];

  // Do the main work of the operation here.
  calculation = 2 * calculation;
    
  [self completeOperation];
  [pool release];
}

- (void) start
{
  NSAutoreleasePool* pool = [[NSAutoreleasePool alloc] init];

  // Always check for cancellation before launching the task.
  if ([self isCancelled])
    {
      // Must move the operation to the finished state if it is canceled.
      [self willChangeValueForKey: @"isFinished"];
      finished = YES;
      [self didChangeValueForKey: @"isFinished"];
      return;
    }
  
  // As the operation is not canceled, begin executing the task.
  [NSThread detachNewThreadSelector: @selector(main)
                           toTarget: self
                         withObject: nil];
  [pool release];
}

- (void) beginOperation
{
  // Also trigger KVO notification for isFinished to ensure its handled
  // correctly if operation is not yet finished
  [self willChangeValueForKey: @"isFinished"];
  [self willChangeValueForKey: @"isExecuting"];
  
  executing = YES;
  finished = NO;
  
  [self didChangeValueForKey: @"isExecuting"];
  [self didChangeValueForKey: @"isFinished"];
}

- (void) completeOperation
{
  [self willChangeValueForKey: @"isFinished"];
  [self willChangeValueForKey: @"isExecuting"];
  
  executing = NO;
  finished = YES;
  
  [self didChangeValueForKey: @"isExecuting"];
  [self didChangeValueForKey: @"isFinished"];
}

@end


int main()
{
  id                    obj;
  NSOperationQueue      *q;
  int                   i;
  NSMutableArray        *a;
# if __has_feature(blocks)
  __block BOOL blockDidRun = NO;
#endif

  START_SET("concurrent operations")

  // single concurrent operation
  obj = [[MyOperation alloc] initWithValue: 1];
# if __has_feature(blocks)
  [obj setCompletionBlock: ^(void){blockDidRun = YES;}];
# endif
  q = [NSOperationQueue new];
  [q addOperation: obj];
  [q waitUntilAllOperationsAreFinished];
  PASS(([obj isFinished] == YES), "operation ran");
  PASS(([obj isExecuting] == NO), "operation is not executing");
# if __has_feature(blocks)
  PASS(blockDidRun == YES, "completion block is executed");
# endif
  PASS(([obj getCalculation] == 2), "operation was performed");
  [obj release];

  // multiple concurrent operations
  [q setMaxConcurrentOperationCount: 10];
  a = [NSMutableArray array];
  for (i = 0; i < 5; ++i)
    {
      obj = [[MyOperation alloc] initWithValue: i];
      [a addObject: obj];
      [q addOperation: obj];
    }
  [q waitUntilAllOperationsAreFinished];
  PASS(([obj isFinished] == YES), "operation ran");
  PASS(([obj isExecuting] == NO), "operation is not executing");

  for (i = 0; i < 5; ++i)
    {
      obj = [a objectAtIndex: i];
      PASS(([obj getCalculation] == (2*i)), "operation was performed");
    }

  // multiple concurrent operations
  [q setMaxConcurrentOperationCount: 5];
  a = [NSMutableArray array];
  for (i = 0; i < 10; ++i)
    {
      obj = [[MyOperation alloc] initWithValue: i];
      [a addObject: obj];
      [obj release];
    }
  [q addOperations: a waitUntilFinished: YES];
  PASS(([obj isFinished] == YES), "operation ran");
  PASS(([obj isExecuting] == NO), "operation is not executing");

  for (i = 0; i < 10; ++i)
    {
      obj = [a objectAtIndex: i];
      PASS(([obj getCalculation] == (2*i)), "operation was performed");
    }

  END_SET("concurrent operations")
  return 0;
}

==> gnustep-base_1.31.1.orig.tar.gz: gnustep-base-1.31.1/Tests/base/NSOperation/concurrent.m <==
#import <Foundation/NSArray.h>
#import <Foundation/NSKeyValueObserving.h>
#import <Foundation/NSLock.h>
#import <Foundation/NSOperation.h>
#import <Foundation/NSThread.h>
#import <Foundation/NSNotification.h>
#import <Foundation/NSAutoreleasePool.h>
#import "ObjectTesting.h"

// concurrent operation
@interface MyOperation : NSOperation
{
  int calculation;

  BOOL executing;
  BOOL finished;
}
- (void) completeOperation;
@end

@implementation MyOperation

- (id) initWithValue: (int)aVal
{
  self = [super init];
  if (self) {
    calculation = aVal;
    executing = NO;
    finished = NO;
  }
  return self;
}

- (BOOL) isConcurrent { return YES; }
- (BOOL) isExecuting { return executing; }
- (BOOL) isFinished { return finished; }
- (int) getCalculation { return calculation; }

- (void) main
{
  NSAutoreleasePool* pool = [[NSAutoreleasePool alloc] init];

  [self beginOperation];

  // Do the main work of the operation here.
  calculation = 2 * calculation;
    
  [self completeOperation];
  [pool release];
}

- (void) start
{
  NSAutoreleasePool* pool = [[NSAutoreleasePool alloc] init];

  // Always check for cancellation before launching the task.
  if ([self isCancelled])
    {
      // Must move the operation to the finished state if it is canceled.
      [self willChangeValueForKey: @"isFinished"];
      finished = YES;
      [self didChangeValueForKey: @"isFinished"];
      return;
    }
  
  // As the operation is not canceled, begin executing the task.
  [NSThread detachNewThreadSelector: @selector(main)
                           toTarget: self
                         withObject: nil];
  [pool release];
}

- (void) beginOperation
{
  // Also trigger KVO notification for isFinished to ensure its handled
  // correctly if operation is not yet finished
  [self willChangeValueForKey: @"isFinished"];
  [self willChangeValueForKey: @"isExecuting"];
  
  executing = YES;
  finished = NO;
  
  [self didChangeValueForKey: @"isExecuting"];
  [self didChangeValueForKey: @"isFinished"];
}

- (void) completeOperation
{
  [self willChangeValueForKey: @"isFinished"];
  [self willChangeValueForKey: @"isExecuting"];
  
  executing = NO;
  finished = YES;
  
  [self didChangeValueForKey: @"isExecuting"];
  [self didChangeValueForKey: @"isFinished"];
}

@end


int main()
{
  id                    obj;
  NSOperationQueue      *q;
  int                   i;
  NSMutableArray        *a;
  NSTimeInterval	s;
  NSTimeInterval	f;
  NSUInteger		ran;
  NSUInteger		want;
# if __has_feature(blocks)
  __block BOOL blockDidRun = NO;
#endif

  START_SET("concurrent operations")

  // single concurrent operation
  obj = [[MyOperation alloc] initWithValue: 1];
# if __has_feature(blocks)
  [obj setCompletionBlock: ^(void){blockDidRun = YES;}];
# endif
  q = [NSOperationQueue new];
  [q addOperation: obj];
  [q waitUntilAllOperationsAreFinished];
  PASS(([obj isFinished] == YES), "operation ran");
  PASS(([obj isExecuting] == NO), "operation is not executing");
# if __has_feature(blocks)
  PASS(blockDidRun == YES, "completion block is executed");
# endif
  PASS(([obj getCalculation] == 2), "operation was performed");
  [obj release];

  // multiple concurrent operations
  s = [NSDate timeIntervalSinceReferenceDate];
  [q setMaxConcurrentOperationCount: 10];
  a = [NSMutableArray array];
  want = 200;
  ran = 0;
  for (i = 0; i < want; ++i)
    {
      obj = [[MyOperation alloc] initWithValue: i];
      [a addObject: obj];
      RELEASE(obj);
      [q addOperation: obj];
    }
  [q waitUntilAllOperationsAreFinished];
  f = [NSDate timeIntervalSinceReferenceDate];
  PASS(([obj isFinished] == YES), "operation ran");
  PASS(([obj isExecuting] == NO), "operation is not executing");

  for (i = 0; i < want; ++i)
    {
      obj = [a objectAtIndex: i];
      if ([obj getCalculation] == (2*i))
	{
	  ran++;
	}
    }
  PASS((ran == want), "many operations, all were performed")
  NSLog(@"Duration for %d concurrent operations %g seconds.", want, (f - s));

  // multiple concurrent operations
  [q setMaxConcurrentOperationCount: 5];
  a = [NSMutableArray array];
  for (i = 0; i < 10; ++i)
    {
      obj = [[MyOperation alloc] initWithValue: i];
      [a addObject: obj];
      [obj release];
    }
  [q addOperations: a waitUntilFinished: YES];
  PASS(([obj isFinished] == YES), "operation ran");
  PASS(([obj isExecuting] == NO), "operation is not executing");

  for (i = 0; i < 10; ++i)
    {
      obj = [a objectAtIndex: i];
      PASS(([obj getCalculation] == (2*i)), "operation was performed");
    }

  RELEASE(q);
  END_SET("concurrent operations")
  return 0;
}
